@@ -33,10 +33,8 @@
 //-----------------------------------------------------------------------------
 #include "util_thread.h"
 #include "util_buffers.h"
+#include "util_platforms.h"
 #include "chuck_errmsg.h"
-#ifndef __PLATFORM_WIN32__
-#include <unistd.h> // usleep
-#endif
 
 
 
@@ -67,21 +65,29 @@ XThread::XThread( )
 //-----------------------------------------------------------------------------
 XThread::~XThread( )
 {
-#if defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__)
-	// cant terminate self
-	bool is_self_thread = (thread != pthread_self());
-#elif defined(__PLATFORM_WIN32__)
-	// ignore for now
-	bool is_self_thread = false;
+#if defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__)
+    // can't self-terminate (much like the Terminator)
+    bool is_self_thread = (thread == pthread_self());
+#elif defined(__PLATFORM_WINDOWS__)
+    // ignore for now
+    bool is_self_thread = false;
 #endif
 
-    if( thread != 0 && !is_self_thread)
+    // if our thread not NULL and this destructor is NOT being called on it
+    if( thread != 0 && !is_self_thread )
     {
-#if defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__)
+        // TODO: find an alternative for Android?
+#if defined(__PLATFORM_APPLE__) || ( defined(__PLATFORM_LINUX__) && !defined(__ANDROID__) ) || defined(__WINDOWS_PTHREAD__)
+        // log
+        EM_log( CK_LOG_FINER, "cancelling thread [0x%x] from [0x%x]...", getID(thread), getID(pthread_self()) );
         pthread_cancel(thread);
         pthread_join(thread, NULL);
-#elif defined(__PLATFORM_WIN32__)
-        TerminateThread((HANDLE)thread, 0);
+        // log
+        EM_log( CK_LOG_FINER, "joined with thread [0x%x] from [0x%x]", getID(thread), getID(pthread_self()) );
+#elif defined(__PLATFORM_WINDOWS__)
+        // log
+        EM_log( CK_LOG_FINER, "terminating thread [0x%x] from [0x%x]...", getID(thread), (t_CKUINT)GetCurrentThreadId() );
+        TerminateThread( (HANDLE)thread, 0 );
 #endif
     }
 }
@@ -96,14 +102,21 @@ XThread::~XThread( )
 bool XThread::start( THREAD_FUNCTION routine, void * ptr )
 {
     bool result = false;
-    
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+
+#if ( defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
     if( pthread_create( &thread, NULL, *routine, ptr ) == 0 )
+    {
+        EM_log( CK_LOG_FINER, "starting thread [0x%x] from [0x%x]...", thread, getID(pthread_self()) );
         result = true;
-#elif defined(__PLATFORM_WIN32__)
+    }
+#elif defined(__PLATFORM_WINDOWS__)
     unsigned thread_id;
     thread = _beginthreadex( NULL, 0, routine, ptr, 0, &thread_id );
-    if( thread ) result = true;
+    if( thread )
+    {
+        EM_log( CK_LOG_FINER, "starting thread [0x%x] from [0x%x]...", thread_id, (t_CKUINT)GetCurrentThreadId() );
+        result = true;
+    }
 #endif
     return result;
 }
@@ -118,11 +131,14 @@ bool XThread::start( THREAD_FUNCTION routine, void * ptr )
 bool XThread::wait( long milliseconds, bool cancel )
 {
     bool result = false;
-    
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
-    if(cancel) pthread_cancel(thread);
-    pthread_join(thread, NULL);
-#elif defined(__PLATFORM_WIN32__)
+
+    // TODO: find an alternative for Android?
+#if( defined(__PLATFORM_APPLE__) || ( defined(__PLATFORM_LINUX__) && !defined(__ANDROID__) ) || defined(__WINDOWS_PTHREAD__) )
+    // cancel the thread?
+    if( cancel ) pthread_cancel( thread );
+    // wait for the thread to terminate
+    pthread_join( thread, NULL );
+#elif defined(__PLATFORM_WINDOWS__)
     DWORD timeout, retval;
     if( milliseconds < 0 ) timeout = INFINITE;
     else timeout = milliseconds;
@@ -146,7 +162,8 @@ bool XThread::wait( long milliseconds, bool cancel )
 //-----------------------------------------------------------------------------
 void XThread :: test( )
 {
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+    // TODO: find an alternative for Android?
+#if ( defined(__PLATFORM_APPLE__) || ( defined(__PLATFORM_LINUX__) && !defined(__ANDROID__) ) || defined(__WINDOWS_PTHREAD__) )
     pthread_testcancel();
 #endif
 }
@@ -154,17 +171,33 @@ void XThread :: test( )
 
 
 
+//-----------------------------------------------------------------------------
+// name: getID() | 1.5.0.4 (ge) added
+// desc: get thread id from a thread handle
+//-----------------------------------------------------------------------------
+t_CKUINT XThread::getID( THREAD_HANDLE t )
+{
+#if ( defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+    return (t_CKUINT)t;
+#elif defined(__PLATFORM_WINDOWS__)
+    return (t_CKUINT)GetThreadId( (HANDLE)t );
+#endif
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: XMutex()
 // desc: ...
 //-----------------------------------------------------------------------------
 XMutex::XMutex( )
 {
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+#if ( defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
     pthread_mutex_init(&mutex, NULL);
-#elif defined(__PLATFORM_WIN32__)
+#elif defined(__PLATFORM_WINDOWS__)
     InitializeCriticalSection(&mutex);
-#endif 
+#endif
 }
 
 
@@ -176,11 +209,11 @@ XMutex::XMutex( )
 //-----------------------------------------------------------------------------
 XMutex::~XMutex( )
 {
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+#if ( defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
     pthread_mutex_destroy( &mutex );
-#elif defined(__PLATFORM_WIN32__)
+#elif defined(__PLATFORM_WINDOWS__)
     DeleteCriticalSection(&mutex);
-#endif 
+#endif
 }
 
 
@@ -192,11 +225,11 @@ XMutex::~XMutex( )
 //-----------------------------------------------------------------------------
 void XMutex::acquire( )
 {
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+#if ( defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
     pthread_mutex_lock(&mutex);
-#elif defined(__PLATFORM_WIN32__)
+#elif defined(__PLATFORM_WINDOWS__)
     EnterCriticalSection(&mutex);
-#endif 
+#endif
 }
 
 
@@ -208,11 +241,11 @@ void XMutex::acquire( )
 //-----------------------------------------------------------------------------
 void XMutex::release( )
 {
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+#if ( defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
     pthread_mutex_unlock(&mutex);
-#elif defined(__PLATFORM_WIN32__)
+#elif defined(__PLATFORM_WINDOWS__)
     LeaveCriticalSection(&mutex);
-#endif 
+#endif
 }
 
 
@@ -226,11 +259,11 @@ XWriteThread * XWriteThread::shared()
 {
     // static XWriteThread s_defaultWriteThread;
 
-	// check
-	if( o_defaultWriteThread == NULL )
-		o_defaultWriteThread = new XWriteThread();
-    
-	return o_defaultWriteThread;
+    // check
+    if( o_defaultWriteThread == NULL )
+        o_defaultWriteThread = new XWriteThread();
+
+    return o_defaultWriteThread;
 }
 
 
@@ -262,9 +295,9 @@ XWriteThread::XWriteThread(size_t data_buffer_size, size_t msg_buffer_size)
 //-----------------------------------------------------------------------------
 XWriteThread::~XWriteThread()
 {
-    SAFE_DELETE( m_msg_buffer );
-    SAFE_DELETE( m_data_buffer );
-    SAFE_DELETE_ARRAY( m_thread_buffer );
+    CK_SAFE_DELETE( m_msg_buffer );
+    CK_SAFE_DELETE( m_data_buffer );
+    CK_SAFE_DELETE_ARRAY( m_thread_buffer );
 }
 
 
@@ -276,11 +309,20 @@ XWriteThread::~XWriteThread()
 //-----------------------------------------------------------------------------
 void XWriteThread::shutdown()
 {
+    // put shutdown message into message buffer
     Message msg;
     msg.operation = Message::SHUTDOWN;
     m_msg_buffer->put( msg );
-    
+
+    // smart log push
+    SmartPushLog logPush;
+
+    // log
+    EM_log( CK_LOG_FINER, "waiting on write thread [0x%x]...", m_thread.getID() );
+    // wait on the thread
     m_thread.wait( -1, false );
+    // log
+    EM_log( CK_LOG_FINER, "done waiting for thread [0x%x]", m_thread.getID() );
 }
 
 
@@ -294,19 +336,19 @@ size_t XWriteThread::fwrite(const void * ptr, size_t size, size_t nitems, FILE *
 {
     if(stream != m_stream)
         flush_data_buffer();
-    
+
     // TODO: overflow detection
-    if(m_data_buffer->put((char*)ptr, size*nitems) == 0)
+    if( m_data_buffer->put((char*)ptr, size*nitems) == 0 )
     {
-        EM_log(CK_LOG_SEVERE, "XWriteThread::fwrite: data buffer overflow");
+        EM_log( CK_LOG_SEVERE, "XWriteThread::fwrite(): data buffer overflow!" );
     }
-    
+
     m_bytes_in_buffer += size*nitems;
     m_stream = stream;
-    
+
     if(m_bytes_in_buffer >= PRODUCER_BUFFER_SIZE)
         flush_data_buffer();
-    
+
     return nitems;
 }
 
@@ -320,14 +362,14 @@ size_t XWriteThread::fwrite(const void * ptr, size_t size, size_t nitems, FILE *
 int XWriteThread::fseek(FILE *stream, long offset, int whence)
 {
     flush_data_buffer();
-    
+
     Message msg;
     msg.file = stream;
     msg.operation = Message::SEEK;
     msg.seek.offset = offset;
     msg.seek.whence = whence;
     m_msg_buffer->put(msg);
-    
+
     return 0;
 }
 
@@ -341,12 +383,12 @@ int XWriteThread::fseek(FILE *stream, long offset, int whence)
 int XWriteThread::fflush(FILE *stream)
 {
     flush_data_buffer();
-    
+
     Message msg;
     msg.file = stream;
     msg.operation = Message::FLUSH;
     m_msg_buffer->put(msg);
-    
+
     return 0;
 }
 
@@ -360,12 +402,12 @@ int XWriteThread::fflush(FILE *stream)
 int XWriteThread::fclose(FILE *stream)
 {
     flush_data_buffer();
-    
+
     Message msg;
     msg.file = stream;
     msg.operation = Message::CLOSE;
     m_msg_buffer->put(msg);
-    
+
     return 0;
 }
 
@@ -385,7 +427,7 @@ void XWriteThread::flush_data_buffer()
         msg.operation = Message::WRITE;
         msg.write.data_size = m_bytes_in_buffer;
         m_msg_buffer->put(msg);
-        
+
         m_bytes_in_buffer = 0;
     }
 }
@@ -397,15 +439,15 @@ void XWriteThread::flush_data_buffer()
 // name: write_cb()
 // desc: thread function
 //-----------------------------------------------------------------------------
-#if ( defined(__PLATFORM_MACOSX__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
+#if ( defined(__PLATFORM_APPLE__) || defined(__PLATFORM_LINUX__) || defined(__WINDOWS_PTHREAD__) )
 void * XWriteThread::write_cb(void * _thiss)
-#elif defined(__PLATFORM_WIN32__)
+#elif defined(__PLATFORM_WINDOWS__)
 unsigned XWriteThread::write_cb(void * _thiss)
 #endif
 {
     XWriteThread * _this = (XWriteThread *) _thiss;
     Message msg;
-    
+
     while(!_this->m_thread_exit)
     {
         while(_this->m_msg_buffer->get(msg))
@@ -416,10 +458,10 @@ unsigned XWriteThread::write_cb(void * _thiss)
                                                                msg.write.data_size);
                 if(actual_size != msg.write.data_size)
                 {
-                    EM_log(CK_LOG_SEVERE, "XWriteThread: buffered data size mismatch (%li : %li)",
-                           msg.write.data_size, actual_size);
+                    EM_log( CK_LOG_SEVERE, "XWriteThread: buffered data size mismatch (%li : %li)",
+                            msg.write.data_size, actual_size );
                 }
-                
+
                 ::fwrite(_this->m_thread_buffer, 1, actual_size, msg.file);
             }
             else if(msg.operation == Message::SEEK)
@@ -440,20 +482,20 @@ unsigned XWriteThread::write_cb(void * _thiss)
                 break;
             }
         }
-        
-        usleep(1000);
+
+        ck_usleep(1000);
     }
-    
+
     delete _this;
     _thiss = _this = NULL;
-    
+
     return 0;
 }
 
 
 
 
-#ifdef __MACOSX_CORE__
+#ifdef __PLATFORM_APPLE__
 t_CKINT XThreadUtil::our_priority = 85;
 #else
 t_CKINT XThreadUtil::our_priority = 0x7fffffff;
@@ -462,7 +504,7 @@ t_CKINT XThreadUtil::our_priority = 0x7fffffff;
 
 
 
-#if !defined(__PLATFORM_WIN32__) || defined(__WINDOWS_PTHREAD__)
+#if !defined(__PLATFORM_WINDOWS__) || defined(__WINDOWS_PTHREAD__)
 //-----------------------------------------------------------------------------
 // name: set_priority()
 // desc: set thread priority
@@ -471,22 +513,37 @@ t_CKBOOL XThreadUtil::set_priority( CHUCK_THREAD tid, t_CKINT priority )
 {
     struct sched_param param;
     int policy;
-    
+
     // log
-    EM_log( CK_LOG_FINE, "setting thread priority to: %ld...", priority );
-    
+    EM_log( CK_LOG_INFO, "setting thread [0x%x] priority to: %ld...", (t_CKUINT)tid, priority );
+
     // get for thread
     if( pthread_getschedparam( tid, &policy, &param) )
         return FALSE;
-    
-    // priority
-    param.sched_priority = priority;
+
+    // pthread_attr_t attr;
+    // pthread_attr_init( &attr );
+    // pthread_attr_setinheritsched( &attr, PTHREAD_EXPLICIT_SCHED );
+
     // policy
     policy = SCHED_RR;
+    // priority
+    param.sched_priority = (int)priority;
     // set for thread, pthread style
     if( pthread_setschedparam( tid, policy, &param ) )
         return FALSE;
-    
+
+    // get for thread
+    if( pthread_getschedparam( tid, &policy, &param) )
+        return FALSE;
+
+    // push
+    EM_pushlog();
+    // log
+    EM_log( CK_LOG_FINE, "verifying thread priority: %ld...", param.sched_priority );
+    // pop
+    EM_poplog();
+
     return TRUE;
 }
 //-----------------------------------------------------------------------------
@@ -499,6 +556,14 @@ t_CKBOOL XThreadUtil::set_priority( t_CKINT priority )
     set_priority( pthread_self(), priority );
     return TRUE;
 }
+t_CKINT XThreadUtil::get_max_priority()
+{
+    return sched_get_priority_max( SCHED_RR );
+}
+t_CKINT XThreadUtil::get_min_priority()
+{
+    return sched_get_priority_min( SCHED_RR );
+}
 #else
 //-----------------------------------------------------------------------------
 // name: set_priority()
@@ -508,14 +573,14 @@ t_CKBOOL XThreadUtil::set_priority( CHUCK_THREAD tid, t_CKINT priority )
 {
     // if priority is 0 then done
     if( !priority ) return TRUE;
-    
+
     // log
-    EM_log( CK_LOG_FINE, "setting thread priority to: %ld...", priority );
-    
+    EM_log( CK_LOG_FINE, "setting thread [0x%x] priority to: %ld...", (t_CKUINT)GetThreadId(tid), priority );
+
     // set the priority the thread, windows style
     if( !SetThreadPriority( tid, priority ) )
         return FALSE;
-    
+
     return TRUE;
 }
 //-----------------------------------------------------------------------------
@@ -528,4 +593,13 @@ t_CKBOOL XThreadUtil::set_priority( t_CKINT priority )
     set_priority( GetCurrentThread(), priority );
     return TRUE;
 }
+t_CKINT XThreadUtil::get_max_priority()
+{
+    return THREAD_PRIORITY_HIGHEST; // THREAD_PRIORITY_TIME_CRITICAL;
+}
+t_CKINT XThreadUtil::get_min_priority()
+{
+    return THREAD_PRIORITY_LOWEST;
+}
 #endif
+
